@@ -45,24 +45,24 @@ public:
       std::string status_variable_name, bool sample_with_replacement,
       const std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
       std::string case_weights_file, bool predict_all, double sample_fraction, double alpha, double minprop,
-      bool holdout, PredictionType prediction_type, uint num_random_splits, uint max_depth,
+      double poisson_tau, bool holdout, PredictionType prediction_type, uint num_random_splits, uint max_depth,
       const std::vector<double>& regularization_factor, bool regularization_usedepth);
   void initR(std::unique_ptr<Data> input_data, uint mtry, uint num_trees, std::ostream* verbose_out, uint seed,
-      uint num_threads, ImportanceMode importance_mode, uint min_node_size, uint min_bucket,
+      uint num_threads, ImportanceMode importance_mode, std::vector<uint>& min_node_size, std::vector<uint>& min_bucket,
       std::vector<std::vector<double>>& split_select_weights,
       const std::vector<std::string>& always_split_variable_names, bool prediction_mode, bool sample_with_replacement,
       const std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
       std::vector<double>& case_weights, std::vector<std::vector<size_t>>& manual_inbag, bool predict_all,
-      bool keep_inbag, std::vector<double>& sample_fraction, double alpha, double minprop, bool holdout,
+      bool keep_inbag, std::vector<double>& sample_fraction, double alpha, double minprop, double poisson_tau, bool holdout,
       PredictionType prediction_type, uint num_random_splits, bool order_snps, uint max_depth,
       const std::vector<double>& regularization_factor, bool regularization_usedepth,
       bool node_stats);
   void init(std::unique_ptr<Data> input_data, uint mtry, std::string output_prefix,
-      uint num_trees, uint seed, uint num_threads, ImportanceMode importance_mode, uint min_node_size, uint min_bucket,
+      uint num_trees, uint seed, uint num_threads, ImportanceMode importance_mode, std::vector<uint>& min_node_size, std::vector<uint>& min_bucket,
       bool prediction_mode, bool sample_with_replacement, const std::vector<std::string>& unordered_variable_names,
       bool memory_saving_splitting, SplitRule splitrule, bool predict_all, std::vector<double>& sample_fraction,
-      double alpha, double minprop, bool holdout, PredictionType prediction_type, uint num_random_splits,
-      bool order_snps, uint max_depth, const std::vector<double>& regularization_factor, bool regularization_usedepth, 
+      double alpha, double minprop, double poisson_tau, bool holdout, PredictionType prediction_type, uint num_random_splits,
+      bool order_snps, uint max_depth, const std::vector<double>& regularization_factor, bool regularization_usedepth,
       bool node_stats);
   virtual void initInternal() = 0;
 
@@ -119,10 +119,10 @@ public:
   uint getMtry() const {
     return mtry;
   }
-  uint getMinNodeSize() const {
+  const std::vector<uint>& getMinNodeSize() const {
     return min_node_size;
   }
-  uint getMinBucket() const {
+  const std::vector<uint>& getMinBucket() const {
     return min_bucket;
   }
   size_t getNumIndependentVariables() const {
@@ -147,6 +147,7 @@ public:
   
   std::vector<std::vector<size_t>> getNumSamplesNodes() {
     std::vector<std::vector<size_t>> result;
+    result.reserve(trees.size());
     for (auto& tree : trees) {
       result.push_back(tree->getNumSamplesNodes());
     }
@@ -154,6 +155,7 @@ public:
   }
   std::vector<std::vector<double>> getNodePredictions() {
     std::vector<std::vector<double>> result;
+    result.reserve(trees.size());
     for (auto& tree : trees) {
       result.push_back(tree->getNodePredictions());
     }
@@ -161,6 +163,7 @@ public:
   }
   std::vector<std::vector<double>> getSplitStats() {
     std::vector<std::vector<double>> result;
+    result.reserve(trees.size());
     for (auto& tree : trees) {
       result.push_back(tree->getSplitStats());
     }
@@ -209,8 +212,8 @@ protected:
   std::vector<std::string> dependent_variable_names; // time,status for survival
   size_t num_trees;
   uint mtry;
-  uint min_node_size;
-  uint min_bucket;
+  std::vector<uint> min_node_size;
+  std::vector<uint> min_bucket;
   size_t num_independent_variables;
   uint seed;
   size_t num_samples;
@@ -231,6 +234,9 @@ protected:
   // MAXSTAT splitrule
   double alpha;
   double minprop;
+  
+  // POISSON splitrule
+  double poisson_tau;
 
   // Multithreading
   uint num_threads;
