@@ -1032,10 +1032,15 @@ void TreeClassification::findBestSplitValueNanSmallQ(size_t nodeID, size_t varID
     }
     
     // Decrease of impurity
-    decrease = sum_right / (double) n_right + sum_left / (double) n_left;
     decrease_nanleft = sum_right / (double) n_right + sum_left_withnan / (double) (n_left + num_samples_node_nan);
     decrease_nanright = sum_right_withnan / (double) (n_right + num_samples_node_nan) + sum_left / (double) n_left;
   
+    if (decrease_nanright > decrease_nanleft) {
+      decrease = decrease_nanright;
+    } else {
+      decrease = decrease_nanleft;
+    }
+  
     // Stop if class-wise minimal bucket size reached
     if (min_bucket->size() > 1) {
       bool stop = false;
@@ -1160,10 +1165,15 @@ void TreeClassification::findBestSplitValueNanLargeQ(size_t nodeID, size_t varID
     }
     
     // Decrease of impurity
-    decrease = sum_right / (double) n_right + sum_left / (double) n_left;
     decrease_nanleft = sum_right / (double) n_right + sum_left_withnan / (double) (n_left + num_samples_node_nan);
     decrease_nanright = sum_right_withnan / (double) (n_right + num_samples_node_nan) + sum_left / (double) n_left;
     
+    if (decrease_nanright > decrease_nanleft) {
+      decrease = decrease_nanright;
+    } else {
+      decrease = decrease_nanleft;
+    }
+    
     // Stop if class-wise minimal bucket size reached
     if (min_bucket->size() > 1) {
       bool stop = false;
